'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _runtime = require('../../errors/runtime');

var _message = require('../../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _handleTagArgs = require('../../utils/handle-tag-args');

var _handleTagArgs2 = _interopRequireDefault(_handleTagArgs);

var _wrapTestFunction = require('./wrap-test-function');

var _wrapTestFunction2 = _interopRequireDefault(_wrapTestFunction);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PROTOCOL_RE = /^https?:\/\//;
var IMPLICIT_PROTOCOL_RE = /^\/\//;

var Fixture = function () {
    function Fixture(name, filename) {
        (0, _classCallCheck3.default)(this, Fixture);

        var nameType = typeof name === 'undefined' ? 'undefined' : (0, _typeof3.default)(name);

        if (nameType !== 'string') throw new _runtime.APIError('fixture', _message2.default.fixtureNameIsNotAString, nameType);

        this.name = name;
        this.path = filename;
        this.pageUrl = 'about:blank';
        this.beforeEachFn = null;
        this.afterEachFn = null;
    }

    Fixture.prototype.page = function page(url) {
        for (var _len = arguments.length, rest = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            rest[_key - 1] = arguments[_key];
        }

        this.pageUrl = (0, _handleTagArgs2.default)(url, rest);

        var urlType = (0, _typeof3.default)(this.pageUrl);

        if (urlType !== 'string') throw new _runtime.APIError('page', _message2.default.fixturePageIsNotAString, urlType);

        if (!PROTOCOL_RE.test(this.pageUrl)) {
            var protocol = IMPLICIT_PROTOCOL_RE.test(this.pageUrl) ? 'http:' : 'http://';

            this.pageUrl = protocol + this.pageUrl;
        }

        return this;
    };

    Fixture.prototype.beforeEach = function beforeEach(fn) {
        var fnType = typeof fn === 'undefined' ? 'undefined' : (0, _typeof3.default)(fn);

        if (fnType !== 'function') throw new _runtime.APIError('beforeEach', _message2.default.beforeEachIsNotAFunction, fnType);

        this.beforeEachFn = (0, _wrapTestFunction2.default)(fn);

        return this;
    };

    Fixture.prototype.afterEach = function afterEach(fn) {
        var fnType = typeof fn === 'undefined' ? 'undefined' : (0, _typeof3.default)(fn);

        if (fnType !== 'function') throw new _runtime.APIError('afterEach', _message2.default.afterEachIsNotAFunction, fnType);

        this.afterEachFn = (0, _wrapTestFunction2.default)(fn);

        return this;
    };

    return Fixture;
}();

exports.default = Fixture;
module.exports = exports['default'];